package com.lqkj.link.module.audit.service;

import com.lqkj.link.module.audit.domain.AuditRecord;
import com.lqkj.link.module.audit.repository.AuditRecordRepository;
import com.lqkj.link.module.authority.domain.UserInfo;
import com.lqkj.link.module.authority.repository.UserInfoRepository;
import com.lqkj.link.module.bulletin.repository.NoticeInfoRepository;
import com.lqkj.link.module.zone.domain.ZoneInfo;
import com.lqkj.link.module.zone.repository.ZoneInfoRepository;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class AuditRecordService {
    private final AuditRecordRepository auditRecordRepository;
    private final ZoneInfoRepository zoneInfoRepository;
    private final UserInfoRepository userInfoRepository;
    private final NoticeInfoRepository noticeInfoRepository;

    public AuditRecordService(AuditRecordRepository auditRecordRepository, ZoneInfoRepository zoneInfoRepository, UserInfoRepository userInfoRepository, NoticeInfoRepository noticeInfoRepository) {
        this.auditRecordRepository = auditRecordRepository;
        this.zoneInfoRepository = zoneInfoRepository;
        this.userInfoRepository = userInfoRepository;
        this.noticeInfoRepository = noticeInfoRepository;
    }

    public Page<Map<String, Object>> pageQuery(String auditor, String author, Integer authStatus, Integer page, Integer pageSize) {
        Pageable pageable = PageRequest.of(page, pageSize);
        return auditRecordRepository.pageQuery(auditor, author, authStatus, pageable);
    }

    @Transactional
    public void delete(List<Integer> recordIds) {
        auditRecordRepository.batchDelete(recordIds);
    }

    @Transactional
    public String updateAuthStatus(Integer recordId, Integer authStatus, String auditor) {
        if (!auditRecordRepository.existsById(recordId)) {
            return "该作品已被作者删除！";
        }
        AuditRecord auditRecord = auditRecordRepository.findById(recordId).get();
        if (auditRecord.getAuthStatus() == 5) {
            return "该作品作者已撤销发布！";
        }

        if (authStatus == 2 || authStatus == 3) {
            if (auditRecord.getAuthStatus() == 2 || auditRecord.getAuthStatus() == 3) {
                return "该作品已被其他审核员完成审核，审核结果：" + (auditRecord.getAuthStatus() == 2 ? "通过" : "未通过");
            }
            auditRecord.setAuthStatus(authStatus);
            UserInfo userInfo = userInfoRepository.findByUserCode(auditor);
            auditRecord.setUserId(userInfo.getUserId());
            auditRecord.setAuthTime(new Date());

            // 向通知添加审核结果
            ZoneInfo zoneInfo = zoneInfoRepository.findById(auditRecord.getZoneId()).get();
            String content = "您的作品【" +
                    zoneInfo.getZoneName() +
                    "】" +
                    (authStatus == 2 ? "审核已通过" : "审核未通过");
            noticeInfoRepository.createAuditNotice(content, zoneInfo.getUserId());

            // 同步更新到作品信息
            zoneInfoRepository.updateAuthStatusWithAuditRecord(recordId, authStatus);
        } else {
            auditRecord.setDeleteTime(new Date());
            auditRecord.setDeleteStatus(true);
        }
        auditRecordRepository.save(auditRecord);
        return null;
    }

    @Transactional
    public String applyOrRevoke(Integer zoneId, Boolean isApply) {
        AuditRecord auditingRecord = auditRecordRepository.findAuditing(zoneId);
        if (isApply) {
            if (auditingRecord != null) {
                return "该作品正在审核中，不能继续申请审核！";
            }
            AuditRecord auditRecord = new AuditRecord();
            auditRecord.setZoneId(zoneId);
            auditRecord.setAuthStatus(1);
            auditRecord.setApplyTime(new Date());
            auditRecord.setDeleteStatus(false);

            auditRecordRepository.save(auditRecord);

            zoneInfoRepository.updateAuthStatus(zoneId, 1);
        } else {
            if (auditingRecord == null) {
                return "该作品未提交审核，不能撤销！";
            }
            auditingRecord.setAuthStatus(5);
            auditRecordRepository.save(auditingRecord);

            zoneInfoRepository.updateAuthStatus(zoneId, 0);
        }

        return null;
    }
}
